import numpy as np
import matplotlib.pyplot as pyplot
from datetime import datetime
import os, calendar

event_types = ['Rain', 'Thunderstorm', 'Snow', 'Fog']
num_events = len(event_types)

def event2int(event):
    return event_types.index(event)

def date2int(date_str):
    date = datetime.strptime(date_str, '%Y-%m-%d')
    return date.toordinal()


#-------------------------------------------------- 

def read_weather(file_name):
    dtypes = np.dtype({ 'names' : ('timestamp', 'max temp', 'mean temp', 'min temp', 'events'),
                       'formats' : [np.int, np.float, np.float, np.float, 'S100'] })

   
    #dtypes =np.dtype([('timestamp',np.int),('max_temp',np.float),('mean_temp',np.float),('min_temp',np.float),('events','S100')])
 

    data = np.loadtxt(file_name, delimiter=',', skiprows=1, 
            converters = { 0 : date2int },
            usecols=(0,1,2,3,21), dtype=dtypes)


    return data

#-------------------------------------------------- 

def temp_plot(dates, tempmean,tempmax,tempmin):

    year_start = datetime(2012, 1, 1)

    days = np.array([(d - year_start).days + 1 for d in dates])
    fig = pyplot.figure()
    fig.add_subplot(2,2,1)
    pyplot.title('Mean Temperatures in Bloomington 2012',size=10)
    pyplot.ylabel('Mean Temperature (F)')
    pyplot.xlabel('Day of Year',size=10)
    pyplot.plot(days, tempmean, marker='o',color='c')
    fig.add_subplot(2,2,2)
    pyplot.title('Max Temperatures in Bloomington 2012',size=10)
    pyplot.ylabel('Max Temperature (F)')
    pyplot.xlabel('Day of Year',size=10)
    pyplot.plot(days, tempmax, marker='o',color='g')
    fig.add_subplot(2,2,3)
    pyplot.title('Min Temperatures in Bloomington 2012',size=10)
    pyplot.ylabel('Min Temperature (F)')
    pyplot.xlabel('Day of Year',size=10)
    pyplot.plot(days, tempmin, marker='o',color='b')
    if not os.path.exists('plots'):
        os.mkdir('plots')
    fig.savefig(os.path.join('plots', 'day_vs_temp.png'))    
    
    
 

def temp_plot_fitted(dates, tempmean,tempmax,tempmin):

    year_start = datetime(2012, 1, 1)

    days = np.array([(d - year_start).days + 1 for d in dates])
    
    fig = pyplot.figure()
    fig.add_subplot(2,2,1)
    pyplot.title('Mean Temperatures in Bloomington 2012',size=10)
    pyplot.ylabel('Mean Temperature (F)')
    pyplot.xlabel('Day of Year',size=10)
    pyplot.plot(days, tempmean,color='c', marker='o')
    slope, intercept = np.polyfit(days, tempmean, 1)
    ideal_temps = intercept + (slope * days)
    fit_label = 'Linear fit ({0:.3f})'.format(slope)
    pyplot.plot(days, ideal_temps, color='red', linestyle='--', label=fit_label)
    pyplot.legend(loc='lower right')
    
    fig.add_subplot(2,2,2)
    pyplot.title('Max Temperatures in Bloomington 2012',size=10)
    pyplot.ylabel('Max Temperature (F)')
    pyplot.xlabel('Day of Year',size=10)
    pyplot.plot(days, tempmax, color='g',marker='o')
    slope, intercept = np.polyfit(days, tempmax, 1)
    ideal_temps = intercept + (slope * days)
    fit_label = 'Linear fit ({0:.3f})'.format(slope)
    pyplot.plot(days, ideal_temps, color='red', linestyle='--', label=fit_label)
    pyplot.legend(loc='lower right')

    fig.add_subplot(2,2,3)
    pyplot.title('Min Temperatures in Bloomington 2012',size=10)
    pyplot.ylabel('Min Temperature (F)')
    pyplot.xlabel('Day of Year',size=10)
    pyplot.plot(days, tempmin ,color='b',marker='o')
    slope, intercept = np.polyfit(days, tempmin, 1)
    ideal_temps = intercept + (slope * days)
    fit_label = 'Linear fit ({0:.3f})'.format(slope)
    pyplot.plot(days, ideal_temps, color='red', linestyle='--', label=fit_label)
    pyplot.legend(loc='lower right')


    if not os.path.exists('plots'):
        os.mkdir('plots')
    fig.savefig(os.path.join('plots', 'day_vs_temp_fitted.png'))
    


#-------------------------------------------------- 

def hist_events(dates, events):
    event_months = [[] for i in xrange(num_events)]
    print event_months

    # Build up lists of months where events occurred
    min_month = 13
    max_month = 0

    for date, event_str in zip(dates, events):
        if len(event_str) == 0:
            continue  # Skip blank events

        month = date.month
        min_month = min(month, min_month)
        max_month = max(month, max_month)

        for event in event_str.split('-'):
            event_code = event2int(event)
            event_months[event_code].append(month)
        
    print event_months
    # Plot histogram
    fig = pyplot.figure()
    pyplot.title('Weather Events in Bloomington 2012')
    pyplot.xlabel('Month')
    pyplot.ylabel('Event Count')
    pyplot.axes().yaxis.grid()

    num_months = max_month - min_month + 1;
    print num_months
    bins = np.arange(1, num_months + 2)  # Need extra bin
    pyplot.hist(event_months, bins=bins, label=event_types)

    # Align month labels to bin centers
    month_names = calendar.month_name[min_month:max_month+1]
    pyplot.xticks(bins + 0.5, month_names)

    pyplot.legend()
    
    return fig

#-------------------------------------------------- 

# Read data and extract dates, temperatures, and events
data = read_weather(os.path.join('data', 'weather.csv'))
min_temps = data['min temp']
mean_temps = data['mean temp']
max_temps = data['max temp']
dates = [datetime.fromordinal(d) for d in data['timestamp']]
events = data['events']

# Do plotting
temp_plot(dates, mean_temps,max_temps,min_temps)
temp_plot_fitted(dates, mean_temps,max_temps,min_temps)


fig = hist_events(dates, events)
fig.savefig(os.path.join('plots', 'event_histogram.png'))
